import Text = Phaser.GameObjects.Text;
import Tween = Phaser.Tweens.Tween;
import BaseLevelScene from './baseLevel.scene';
import { Button, ImageButton, RoundedProgressBar } from '../components';
import { configController, soundsController, usageController } from '../controllers';
import { gameConfig, gameStat, soundConfig } from '../config';
import ClientSprite from '../sprites/client.sprite';
import EffectsContainer from '../sprites/effects.container';
import { clients, effects, ingredients, ingredientsImpact, ingredientsImpactLimit } from '../models';
import IngredientSprite from '../sprites/ingredient.sprite';

export default class GameScene extends BaseLevelScene {

  flipSounds = ['bookFlip2', 'bookFlip3'];
  moneySounds = ['handleCoins', 'handleCoins2'];

  muteButton: ImageButton;

  scoreText: Text;

  clientEffectsContainer: EffectsContainer;
  currentEffectsContainer: EffectsContainer;

  ingredientSprites: IngredientSprite[] = [];
  selectedIngredients: IngredientSprite[] = [];

  sellButton: Button;
  cancelButton: Button;
  skipButton: Button;
  tradeButtons: Button[] = [];

  hideButtonsTween: Tween;
  showButtonsTween: Tween;

  hideIngredientsTween: Tween;
  showIngredientsTween: Tween;

  hideProgressbarTween: Tween;
  showProgressbarTween: Tween;

  targetEffects: string[] = [];
  currentEffects: string[] = [];
  currentEffectsMap: Record<string, number> = {};

  progressbar: RoundedProgressBar;
  isCustomerActive = false;

  constructor() {
    super({ key: 'GameScene' });
  }

  create() {
    this.fadeIn(500);
    this.getWorldView();
    soundsController.playSoundtrack('the-first-snow');
    //monetizationController.mockStart();

    let i;
    this.resetCurrentEffects();

    this.muteButton = new ImageButton(this, 50, 50, 80, 80, 'buttonSquare_grey', soundConfig.soundsMuted ?
      'musicOff' : 'musicOn', () => this.changeMuteState(!soundConfig.soundsMuted));
    this.add.existing(this.muteButton);
    this.fixedContainers.push(this.muteButton);


    this.scoreText = new Text(this, this.view.right - 10, 10,  '', {fontSize: '50px', fontFamily: '"Architects Daughter"', color: 'black'});
    this.scoreText.setOrigin(1, 0);
    this.add.existing(this.scoreText);
    this.updateScore(0);

    this.sellButton = new Button(this, this.view.centerX - 130, this.view.bottom - 625, 100, 100, '$', 'buttonSquare_grey', {fontSize: '60px'}, () => {
      this.closeOrder();
    });
    this.cancelButton = new Button(this, this.view.centerX, this.view.bottom - 625, 100, 100, 'X', 'buttonSquare_grey', {fontSize: '60px'}, () => {
      this.resetPoison();
      soundsController.actionSound(this.flipSounds[Phaser.Math.Between(0, this.flipSounds.length - 1)]);
    });
    this.skipButton = new Button(this, this.view.centerX + 130, this.view.bottom - 625, 100, 100, '<', 'buttonSquare_grey', {fontSize: '70px'}, () => {
      this.skipOrder();
    });
    this.tradeButtons = [this.sellButton, this.cancelButton, this.skipButton];
    this.add.existing(this.sellButton.setAlpha(0));
    this.add.existing(this.cancelButton.setAlpha(0));
    this.add.existing(this.skipButton.setAlpha(0));

    this.hideButtonsTween = this.tweens.add({
      targets: this.tradeButtons,
      alpha: 0,
      duration: 200,
    });
    this.hideButtonsTween.stop(0);
    this.showButtonsTween = this.tweens.add({
      targets: this.tradeButtons,
      alpha: 1,
      duration: 200,
    });
    this.showButtonsTween.stop(0);

    this.client = new ClientSprite(this, 'doctor');
    this.add.existing(this.client);

    this.clientEffectsContainer = new EffectsContainer(this, this.view.centerX, 280);
    this.currentEffectsContainer = new EffectsContainer(this, this.view.centerX, 380);

    for (i = 0; i < 9; i++) {
      this.ingredientSprites.push(new IngredientSprite(this, 'apple', 'soft_gray', s => {this.selectIngredient(s)}));
      this.ingredientSprites[i].setAlpha(0);
    }
    Phaser.Actions.GridAlign(this.ingredientSprites, {
      width: 3,
      height: 3,
      cellWidth: 180,
      cellHeight: 180,
      x: this.view.centerX - 90,
      y: this.view.bottom - 400,
    });
    this.hideIngredientsTween = this.tweens.add({
      targets: this.ingredientSprites,
      alpha: 0,
      duration: 200,
    });
    this.hideIngredientsTween.stop(0);
    this.showIngredientsTween = this.tweens.add({
      targets: this.ingredientSprites,
      alpha: 1,
      duration: 200,
    });
    this.showIngredientsTween.stop(0);

    this.newClient();
    if (!gameStat.tutorialFinished) {
      configController.setConfig(gameStat, 'tutorialFinished', true);
    }

    this.progressbar = new RoundedProgressBar(this, this.view.centerX, 60, 100, 27);
    this.add.existing(this.progressbar);
    this.progressbar.setAlpha(0);
    this.hideProgressbarTween = this.tweens.add({
      targets: this.progressbar,
      alpha: 0,
      duration: 200,
    });
    this.hideProgressbarTween.stop(0);
    this.showProgressbarTween = this.tweens.add({
      targets: this.progressbar,
      alpha: 1,
      duration: 200,
    });
    this.showProgressbarTween.stop(0);
  }


  update(time, delta) {
    super.update(time, delta);
    this.children.each(i => i.update(time, delta));
    if (this.isCustomerActive) {
      this.progressbar.changeProgress(-0.000375);
      if (this.progressbar.progress < 0.001) {
        this.skipOrder();
      }
    }
  }

  onResize() {
    super.onResize();
    this.muteButton?.setPosition(50, 50);
    this.scoreText?.setPosition(this.view.right - 210, this.view.top + 60);
  }



  changeMuteState(mute) {
    if (mute) {
      soundsController.mute();
    } else {
      soundsController.unmute();
    }
    this.muteButton.setImage(mute ? 'musicOff' : 'musicOn');
  }

  updateScore(delta) {
    configController.setConfig(gameStat, 'score', gameStat.score + delta);
    this.scoreText.setText(gameStat.score.toString());
  }

  updateClients() {
    configController.setConfig(gameStat, 'clients', gameStat.clients + 1);
  }

  resetCurrentEffects() {
    let i;
    for (i = 0; i < effects.length; i++) {
      this.currentEffectsMap[effects[i]] = 0;
    }
    this.currentEffects = [];
  }

  addOrder() {
    let i, tries = 15;
    let availableEffects: string[] = [];
    do {
      const localIngredients = Phaser.Utils.Array.Shuffle([...ingredients]).slice(0, 9);
      for (i = 0; i < 9; i++) {
        this.ingredientSprites[i].setType(localIngredients[i]);
      }
      availableEffects = effects
        .filter(e => localIngredients.reduce((a, i) => a + Math.max(ingredientsImpact[i][e], 0), 0) >= ingredientsImpactLimit);
      tries--;
    } while (availableEffects.length < 2 && tries > 0);
    if (tries == 0) {
      this.skipOrder();
      return;
    }
    //console.log(availableEffects);

    const effect = availableEffects[Phaser.Math.Between(0, availableEffects.length - 1)];
    this.targetEffects = [effect];
    this.showIngredientsTween.restart();
    this.progressbar.setProgress(1);
    this.showProgressbarTween.restart();
    this.clientEffectsContainer.setEffects(this.targetEffects);
    this.isCustomerActive = true;
  }

  skipOrder() {
    this.resetPoison();
    this.closeOrder();
  }

  resetPoison() {
    this.resetCurrentEffects();
    this.currentEffectsContainer.setEffects([]);
    let i;
    for (i = 0; i < this.selectedIngredients.length; i++) {
      this.selectedIngredients[i].setBackground('soft_gray');
    }
    this.selectedIngredients = [];
    this.hideButtonsTween.restart();
  }

  closeOrder() {
    this.isCustomerActive = false;
    const score = this.currentEffects
      .filter(e => this.targetEffects.includes(e))
      .reduce((acc, e) => this.currentEffectsMap[e], 0) +
      this.currentEffects
        .filter(e => !this.targetEffects.includes(e)).length * -0.25;
    const resultScore = Math.min(Math.max(Math.floor(score * 1000 / this.selectedIngredients.length), 0), 3000);
    if (resultScore) {
      this.updateScore(resultScore);
      soundsController.actionSound(this.moneySounds[Phaser.Math.Between(0, this.moneySounds.length - 1)]);
    }
    this.updateClients();

    this.clientEffectsContainer.setEffects([]);
    this.resetPoison();
    this.hideProgressbarTween.restart();
    this.hideIngredientsTween.restart();
    this.client.walkTween(this.view.left - 50, 1500, () => {
      this.newClient();
    });
  }

  newClient() {
    let type;
    do {
      type = clients[Phaser.Math.Between(0, clients.length - 1)];
    } while (this.client.type == type);
    this.client.setType(type).setPosition(this.view.right + 45, 150);
    this.client.walkTween(this.view.centerX, 1500, () => {
      this.addOrder();
    });
  }

  selectIngredient(s: IngredientSprite) {
    this.selectedIngredients.push(s);
    let i;
    const impact = ingredientsImpact[s.type];
    for (i = 0; i < effects.length; i++) {
      this.currentEffectsMap[effects[i]] += impact[effects[i]];
    }
    this.currentEffects = effects.filter(e => this.currentEffectsMap[e] >= ingredientsImpactLimit);
    this.currentEffectsContainer.setEffects(this.currentEffects);
    if (this.selectedIngredients.length == 1) {
      this.showButtonsTween.restart();
    }
  }

}
